@@ -22,13 +22,16 @@
  ***************************************************************************/
 
 // qt/kde includes
+#include <qcheckbox.h>
 #include <qsplitter.h>
 #include <qpainter.h>
 #include <qlayout.h>
 #include <qlabel.h>
 #include <qvbox.h>
 #include <qtoolbox.h>
+#include <qtooltip.h>
 #include <qpushbutton.h>
+#include <qwhatsthis.h>
 #include <dcopobject.h>
 #include <dcopclient.h>
 #include <kapplication.h>
@@ -36,6 +39,7 @@
 #include <kdirwatch.h>
 #include <kinstance.h>
 #include <kprinter.h>
+#include <kdeprint/kprintdialogpage.h>
 #include <kstdaction.h>
 #include <kdeversion.h>
 #include <kparts/genericfactory.h>
@@ -70,6 +74,35 @@
 #include "core/document.h"
 #include "core/page.h"
 
+class PDFOptionsPage : public KPrintDialogPage
+{
+   public:
+       PDFOptionsPage()
+       {
+           setTitle( i18n( "PDF Options" ) );
+           QVBoxLayout *layout = new QVBoxLayout(this);
+           m_forceRaster = new QCheckBox(i18n("Force rasterization"), this);
+           QToolTip::add(m_forceRaster, i18n("Rasterize into an image before printing"));
+           QWhatsThis::add(m_forceRaster, i18n("Forces the rasterization of each page into an image before printing it. This usually gives somewhat worse results, but is useful when printing documents that appear to print incorrectly."));
+           layout->addWidget(m_forceRaster);
+           layout->addStretch(1);
+       }
+
+       void getOptions( QMap<QString,QString>& opts, bool incldef = false )
+       {
+           Q_UNUSED(incldef);
+           opts[ "kde-kpdf-forceRaster" ] = QString::number( m_forceRaster->isChecked() );
+       }
+
+       void setOptions( const QMap<QString,QString>& opts )
+       {
+           m_forceRaster->setChecked( opts[ "kde-kpdf-forceRaster" ].toInt() );
+       }
+
+    private:
+        QCheckBox *m_forceRaster;
+};
+
 // definition of searchID for this class
 #define PART_SEARCH_ID 1
 
@@ -89,6 +122,10 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 	// connect the started signal to tell the job the mimetypes we like
 	connect(this, SIGNAL(started(KIO::Job *)), this, SLOT(setMimeTypes(KIO::Job *)));
 	
+	// connect the completed signal so we can put the window caption when loading remote files
+	connect(this, SIGNAL(completed()), this, SLOT(emitWindowCaption()));
+	connect(this, SIGNAL(canceled(const QString &)), this, SLOT(emitWindowCaption()));
+	
 	// load catalog for translation
 	KGlobal::locale()->insertCatalogue("kpdf");
 
@@ -106,7 +143,7 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 	setInstance(KPDFPartFactory::instance());
 
 	// build the document
-	m_document = new KPDFDocument();
+	m_document = new KPDFDocument(widget());
 	connect( m_document, SIGNAL( linkFind() ), this, SLOT( slotFind() ) );
 	connect( m_document, SIGNAL( linkGoToPage() ), this, SLOT( slotGoToPage() ) );
 	connect( m_document, SIGNAL( linkPresentation() ), this, SLOT( slotShowPresentation() ) );
@@ -149,9 +186,13 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 
 	int index;
 	// [left toolbox: Table of Contents] | []
-	m_tocFrame = new TOC( m_toolBox, m_document );
+	// dummy wrapper with layout to enable horizontal scroll bars (bug: 147233)
+	QWidget *tocWrapper = new QWidget(m_toolBox);
+	QVBoxLayout *tocWrapperLayout = new QVBoxLayout(tocWrapper);
+	m_tocFrame = new TOC( tocWrapper, m_document );
+	tocWrapperLayout->add(m_tocFrame);
 	connect(m_tocFrame, SIGNAL(hasTOC(bool)), this, SLOT(enableTOC(bool)));
-	index = m_toolBox->addItem( m_tocFrame, QIconSet(SmallIcon("text_left")), i18n("Contents") );
+	index = m_toolBox->addItem( tocWrapper, QIconSet(SmallIcon("text_left")), i18n("Contents") );
 	m_toolBox->setItemToolTip(index, i18n("Contents"));
 	enableTOC( false );
 
@@ -192,6 +233,7 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 	// widgets: [] | [right 'pageView']
 	m_pageView = new PageView( m_splitter, m_document );
 	m_pageView->setFocus(); //usability setting
+	m_splitter->setFocusProxy(m_pageView);
 	connect( m_pageView, SIGNAL( urlDropped( const KURL& ) ), SLOT( openURLFromDocument( const KURL & )));
 	connect( m_pageView, SIGNAL( rightClick(const KPDFPage *, const QPoint &) ), this, SLOT( slotShowMenu(const KPDFPage *, const QPoint &) ) );
 
@@ -356,7 +398,18 @@ bool Part::openFile()
     KMimeType::Ptr mime;
     if ( m_bExtension->urlArgs().serviceType.isEmpty() )
     {
-        mime = KMimeType::findByPath( m_file );
+        if (!m_jobMime.isEmpty())
+        {
+            mime = KMimeType::mimeType(m_jobMime);
+            if ( mime->is( "application/octet-stream" ) )
+            {
+                mime = KMimeType::findByPath( m_file );
+            }
+        }
+        else
+        {
+            mime = KMimeType::findByPath( m_file );
+        }
     }
     else
     {
@@ -421,6 +474,11 @@ bool Part::openFile()
     if ( !m_watcher->contains(m_file) )
         m_watcher->addFile(m_file);
 
+    // if the 'OpenTOC' flag is set, open the TOC
+    if ( m_document->getMetaData( "OpenTOC" ) == "yes" && m_toolBox->isItemEnabled( 0 ) )
+    {
+        m_toolBox->setCurrentIndex( 0 );
+    }
     // if the 'StartFullScreen' flag is set, start presentation
     if ( m_document->getMetaData( "StartFullScreen" ) == "yes" )
     {
@@ -444,12 +502,22 @@ bool Part::openURL(const KURL &url)
     // if it matches then: download it (if not local) extract to a temp file using
     // KTar and proceed with the URL of the temporary file
 
+    m_jobMime = QString::null;
+
     // this calls the above 'openURL' method
     bool b = KParts::ReadOnlyPart::openURL(url);
+
+    // these setWindowCaption calls only work for local files
     if ( !b )
+    {
         KMessageBox::error( widget(), i18n("Could not open %1").arg( url.prettyURL() ) );
+        emit setWindowCaption("");
+    }
     else
+    {
         m_viewportDirty.pageNumber = -1;
+        emit setWindowCaption(url.filename());
+    }
     emit enablePrintAction(b);
     return b;
 }
@@ -459,9 +527,22 @@ void Part::setMimeTypes(KIO::Job *job)
     if (job)
     {
         job->addMetaData("accept", "application/pdf, */*;q=0.5");
+        connect(job, SIGNAL(mimetype(KIO::Job*,const QString&)), this, SLOT(readMimeType(KIO::Job*,const QString&)));
     }
 }
 
+void Part::readMimeType(KIO::Job *, const QString &mime)
+{
+	m_jobMime = mime;
+}
+
+void Part::emitWindowCaption()
+{
+    // these setWindowCaption call only works for remote files
+    if (m_document->isOpened()) emit setWindowCaption(url().filename());
+    else emit setWindowCaption("");
+}
+
 bool Part::closeURL()
 {
     if (!m_temporaryLocalFile.isNull())
@@ -526,15 +607,23 @@ void Part::slotDoFileDirty()
   if (m_viewportDirty.pageNumber == -1)
   {
     m_viewportDirty = m_document->viewport();
+    m_dirtyToolboxIndex = m_toolBox->currentIndex();
+    m_wasPresentationOpen = ((PresentationWidget*)m_presentationWidget != 0);
     m_pageView->showText(i18n("Reloading the document..."), 0);
   }
 
-  if (KParts::ReadOnlyPart::openURL(m_file))
+  if (KParts::ReadOnlyPart::openURL(KURL::fromPathOrURL(m_file)))
   {
     if (m_viewportDirty.pageNumber >= (int)m_document->pages()) m_viewportDirty.pageNumber = (int)m_document->pages() - 1;
     m_document->setViewport(m_viewportDirty);
     m_viewportDirty.pageNumber = -1;
+    if ( m_toolBox->currentIndex() != m_dirtyToolboxIndex && m_toolBox->isItemEnabled( m_dirtyToolboxIndex ) )
+    {
+      m_toolBox->setCurrentIndex( m_dirtyToolboxIndex );
+    }
+    if (m_wasPresentationOpen) slotShowPresentation();
     emit enablePrintAction(true);
+    emit setWindowCaption(url().filename());
   }
   else
   {
@@ -679,6 +768,7 @@ void Part::slotHistoryNext()
 
 void Part::slotFind()
 {
+    static bool savedCaseSensitive = false;
     KFindDialog dlg( widget() );
     dlg.setHasCursor( false );
     if ( !m_searchHistory.empty() )
@@ -688,12 +778,17 @@ void Part::slotFind()
     dlg.setSupportsWholeWordsFind( false );
     dlg.setSupportsRegularExpressionFind( false );
 #endif
+    if ( savedCaseSensitive )
+    {
+        dlg.setOptions( dlg.options() | KFindDialog::CaseSensitive );
+    }
     if ( dlg.exec() == QDialog::Accepted )
     {
+        savedCaseSensitive = dlg.options() & KFindDialog::CaseSensitive;
         m_searchHistory = dlg.findHistory();
         m_searchStarted = true;
         m_document->resetSearch( PART_SEARCH_ID );
-        m_document->searchText( PART_SEARCH_ID, dlg.pattern(), false, dlg.options() & KFindDialog::CaseSensitive,
+        m_document->searchText( PART_SEARCH_ID, dlg.pattern(), false, savedCaseSensitive,
                                 KPDFDocument::NextMatch, true, qRgb( 255, 255, 64 ) );
     }
 }
@@ -789,7 +884,6 @@ void Part::slotPrintPreview()
 
     printer.setMinMax(1, m_document->pages());
     printer.setPreviewOnly( true );
-    printer.setMargins(0, 0, 0, 0);
 
     // if some pages are landscape and others are not the most common win as kprinter does
     // not accept a per page setting
@@ -900,7 +994,10 @@ void Part::slotShowProperties()
 void Part::slotShowPresentation()
 {
     if ( !m_presentationWidget )
+    {
       m_presentationWidget = new PresentationWidget( widget(), m_document );
+        m_presentationWidget->setupActions( actionCollection() );
+    }
 }
 
 void Part::slotHidePresentation()
@@ -909,6 +1006,19 @@ void Part::slotHidePresentation()
         delete (PresentationWidget*) m_presentationWidget;
 }
 
+void Part::slotTogglePresentation()
+{
+    if ( m_document->isOpened() )
+    {
+        if ( !m_presentationWidget )
+        {
+            m_presentationWidget = new PresentationWidget( widget(), m_document );
+            m_presentationWidget->setupActions( actionCollection() );
+        }
+        else delete (PresentationWidget*) m_presentationWidget;
+    }
+}
+
 void Part::slotPrint()
 {
     if (m_document->pages() == 0) return;
@@ -921,7 +1031,6 @@ void Part::slotPrint()
     printer.setPageSelection(KPrinter::ApplicationSide);
     printer.setMinMax(1, m_document->pages());
     printer.setCurrentPage(m_document->currentPage()+1);
-    printer.setMargins(0, 0, 0, 0);
 
     // if some pages are landscape and others are not the most common win as kprinter does
     // not accept a per page setting
@@ -938,6 +1047,7 @@ void Part::slotPrint()
     }
     if (landscape > portrait) printer.setOrientation(KPrinter::Landscape);
 
+    KPrinter::addDialogPage(new PDFOptionsPage());
     if (printer.setup(widget())) doPrint( printer );
 }
 
