@@ -23,18 +23,15 @@
 #include <ctype.h>
 #include "gmem.h"
 #include "gfile.h"
-#include "xpdf_config.h"
+#include "config.h"
 #include "Error.h"
 #include "Object.h"
 #include "Lexer.h"
-#include "Decrypt.h"
 #include "GfxState.h"
 #include "Stream.h"
 #include "JBIG2Stream.h"
 #include "JPXStream.h"
 #include "Stream-CCITT.h"
-#include "DCTStream.h"
-#include "UGString.h"
 
 #ifdef __DJGPP__
 static GBool setDJSYSFLAGS = gFalse;
@@ -88,7 +85,7 @@ char *Stream::getLine(char *buf, int size) {
   return buf;
 }
 
-GString *Stream::getPSFilter(int /*psLevel*/, const char */*indent*/) {
+GString *Stream::getPSFilter(int /*psLevel*/, char * /*indent*/) {
   return new GString();
 }
 
@@ -136,7 +133,7 @@ Stream *Stream::addFilters(Object *dict) {
   return str;
 }
 
-Stream *Stream::makeFilter(const char *name, Stream *str, Object *params) {
+Stream *Stream::makeFilter(char *name, Stream *str, Object *params) {
   int pred;			// parameters
   int colors;
   int bits;
@@ -144,6 +141,7 @@ Stream *Stream::makeFilter(const char *name, Stream *str, Object *params) {
   int encoding;
   GBool endOfLine, byteAlign, endOfBlock, black;
   int columns, rows;
+  int colorXform;
   Object globals, obj;
 
   if (!strcmp(name, "ASCIIHexDecode") || !strcmp(name, "AHx")) {
@@ -229,7 +227,14 @@ Stream *Stream::makeFilter(const char *name, Stream *str, Object *params) {
     str = new CCITTFaxStream(str, encoding, endOfLine, byteAlign,
 			     columns, rows, endOfBlock, black);
   } else if (!strcmp(name, "DCTDecode") || !strcmp(name, "DCT")) {
-    str = new DCTStream(str);
+    colorXform = -1;
+    if (params->isDict()) {
+      if (params->dictLookup("ColorTransform", &obj)->isInt()) {
+	colorXform = obj.getInt();
+      }
+      obj.free();
+    }
+    str = new DCTStream(str, colorXform);
   } else if (!strcmp(name, "FlateDecode") || !strcmp(name, "Fl")) {
     pred = 1;
     columns = 1;
@@ -275,18 +280,10 @@ Stream *Stream::makeFilter(const char *name, Stream *str, Object *params) {
 
 BaseStream::BaseStream(Object *dictA) {
   dict = *dictA;
-  decrypt = NULL;
 }
 
 BaseStream::~BaseStream() {
   dict.free();
-  if (decrypt)
-    delete decrypt;
-}
-
-void BaseStream::doDecryption(Guchar *fileKey, int keyLength,
-			      int objNum, int objGen) {
-  decrypt = new Decrypt(fileKey, keyLength, objNum, objGen);
 }
 
 //------------------------------------------------------------------------
@@ -412,19 +409,18 @@ StreamPredictor::StreamPredictor(Stream *strA, int predictorA,
   predLine = NULL;
   ok = gFalse;
 
-  if (width <= 0 || nComps <= 0 || nBits <= 0 ||
-      nComps >= INT_MAX / nBits ||
-      width >= INT_MAX / nComps / nBits)
-    return;
-
   nVals = width * nComps;
-  if (nVals * nBits + 7 < 0)
+  if (width <= 0 || nComps <= 0 || nBits <= 0 ||
+      nComps > gfxColorMaxComps || nBits > 16 ||
+      width >= INT_MAX / nComps ||
+      nVals >= (INT_MAX - 7) / nBits) {
     return;
+  }
   pixBytes = (nComps * nBits + 7) >> 3;
   rowBytes = ((nVals * nBits + 7) >> 3) + pixBytes;
-  if (rowBytes <= 0)
+  if (rowBytes <= 0) {
     return;
-
+  }
   predLine = (Guchar *)gmalloc(rowBytes);
   memset(predLine, 0, rowBytes);
   predIdx = rowBytes;
@@ -546,17 +542,17 @@ GBool StreamPredictor::getNextLine() {
       j = k = pixBytes;
       for (i = 0; i < width; ++i) {
 	for (kk = 0; kk < nComps; ++kk) {
-	if (inBits < nBits) {
-	  inBuf = (inBuf << 8) | (predLine[j++] & 0xff);
-	  inBits += 8;
-	}
-	  upLeftBuf[kk] = (upLeftBuf[kk] +
-			(inBuf >> (inBits - nBits))) & bitMask;
-	inBits -= nBits;
+	  if (inBits < nBits) {
+	    inBuf = (inBuf << 8) | (predLine[j++] & 0xff);
+	    inBits += 8;
+	  }
+	  upLeftBuf[kk] = (Guchar)((upLeftBuf[kk] +
+				    (inBuf >> (inBits - nBits))) & bitMask);
+	  inBits -= nBits;
 	  outBuf = (outBuf << nBits) | upLeftBuf[kk];
-	outBits += nBits;
+	  outBits += nBits;
 	  if (outBits >= 8) {
-	  predLine[k++] = (Guchar)(outBuf >> (outBits - 8));
+	    predLine[k++] = (Guchar)(outBuf >> (outBits - 8));
 	    outBits -= 8;
 	  }
 	}
@@ -614,8 +610,6 @@ void FileStream::reset() {
   saved = gTrue;
   bufPtr = bufEnd = buf;
   bufPos = start;
-  if (decrypt)
-    decrypt->reset();
 }
 
 void FileStream::close() {
@@ -633,7 +627,6 @@ void FileStream::close() {
 
 GBool FileStream::fillBuf() {
   int n;
-  char *p;
 
   bufPos += bufEnd - buf;
   bufPtr = bufEnd = buf;
@@ -650,11 +643,6 @@ GBool FileStream::fillBuf() {
   if (bufPtr >= bufEnd) {
     return gFalse;
   }
-  if (decrypt) {
-    for (p = buf; p < bufEnd; ++p) {
-      *p = (char)decrypt->decryptByte((Guchar)*p);
-    }
-  }
   return gTrue;
 }
 
@@ -743,9 +731,6 @@ Stream *MemStream::makeSubStream(Guint startA, GBool limited,
 
 void MemStream::reset() {
   bufPtr = buf + start;
-  if (decrypt) {
-    decrypt->reset();
-  }
 }
 
 void MemStream::close() {
@@ -773,25 +758,6 @@ void MemStream::moveStart(int delta) {
   bufPtr = buf + start;
 }
 
-void MemStream::doDecryption(Guchar *fileKey, int keyLength,
-			     int objNum, int objGen) {
-  char *newBuf;
-  char *p, *q;
-
-  this->BaseStream::doDecryption(fileKey, keyLength, objNum, objGen);
-  if (decrypt) {
-    newBuf = (char *)gmalloc(length);
-    for (p = buf + start, q = newBuf; p < bufEnd; ++p, ++q) {
-      *q = (char)decrypt->decryptByte((Guchar)*p);
-    }
-    bufEnd = newBuf + length;
-    bufPtr = newBuf + (bufPtr - (buf + start));
-    start = 0;
-    buf = newBuf;
-    needFree = gTrue;
-  }
-}
-
 //------------------------------------------------------------------------
 // EmbedStream
 //------------------------------------------------------------------------
@@ -808,7 +774,7 @@ EmbedStream::~EmbedStream() {
 }
 
 Stream *EmbedStream::makeSubStream(Guint /*start*/, GBool /*limitedA*/,
-				   Guint /*lengthA*/, Object */*dictA*/) {
+				   Guint /*lengthA*/, Object * /*dictA*/) {
   error(-1, "Internal: called makeSubStream() on EmbedStream");
   return NULL;
 }
@@ -914,7 +880,7 @@ int ASCIIHexStream::lookChar() {
   return buf;
 }
 
-GString *ASCIIHexStream::getPSFilter(int psLevel, const char *indent) {
+GString *ASCIIHexStream::getPSFilter(int psLevel, char *indent) {
   GString *s;
 
   if (psLevel < 2) {
@@ -995,7 +961,7 @@ int ASCII85Stream::lookChar() {
   return b[index];
 }
 
-GString *ASCII85Stream::getPSFilter(int psLevel, const char *indent) {
+GString *ASCII85Stream::getPSFilter(int psLevel, char *indent) {
   GString *s;
 
   if (psLevel < 2) {
@@ -1181,7 +1147,7 @@ int LZWStream::getCode() {
   return code;
 }
 
-GString *LZWStream::getPSFilter(int psLevel, const char *indent) {
+GString *LZWStream::getPSFilter(int psLevel, char *indent) {
   GString *s;
 
   if (psLevel < 2 || pred) {
@@ -1222,7 +1188,7 @@ void RunLengthStream::reset() {
   eof = gFalse;
 }
 
-GString *RunLengthStream::getPSFilter(int psLevel, const char *indent) {
+GString *RunLengthStream::getPSFilter(int psLevel, char *indent) {
   GString *s;
 
   if (psLevel < 2) {
@@ -1278,16 +1244,16 @@ CCITTFaxStream::CCITTFaxStream(Stream *strA, int encodingA, GBool endOfLineA,
   byteAlign = byteAlignA;
   columns = columnsA;
   if (columns < 1) {
-     columns = 1;
+    columns = 1;
   }
   if (columns + 4 <= 0) {
-     columns = INT_MAX - 4;
+    columns = INT_MAX - 4;
   }
   rows = rowsA;
   endOfBlock = endOfBlockA;
   black = blackA;
-  refLine = (short *)gmallocn(columns + 4, sizeof(short));
-  codingLine = (short *)gmallocn(columns + 3, sizeof(short));
+  refLine = (short *)gmallocn(columns + 3, sizeof(short));
+  codingLine = (short *)gmallocn(columns + 2, sizeof(short));
 
   eof = gFalse;
   row = 0;
@@ -1315,7 +1281,7 @@ void CCITTFaxStream::reset() {
   nextLine2D = encoding < 0;
   inputBits = 0;
   codingLine[0] = 0;
-  codingLine[1] = refLine[2] = columns;
+  codingLine[1] = columns;
   a0 = 1;
   buf = EOF;
 
@@ -1351,8 +1317,27 @@ int CCITTFaxStream::lookChar() {
 
     // 2-D encoding
     if (nextLine2D) {
-      for (i = 0; codingLine[i] < columns; ++i)
+      // state:
+      //   a0New = current position in coding line (0 <= a0New <= columns)
+      //   codingLine[a0] = last change in coding line
+      //                    (black-to-white if a0 is even,
+      //                     white-to-black if a0 is odd)
+      //   refLine[b1] = next change in reference line of opposite color
+      //                 to a0
+      // invariants:
+      //   0 <= codingLine[a0] <= a0New
+      //           <= refLine[b1] <= refLine[b1+1] <= columns
+      //   0 <= a0 <= columns+1
+      //   refLine[0] = 0
+      //   refLine[n] = refLine[n+1] = columns
+      //     -- for some 1 <= n <= columns+1
+      // end condition:
+      //   0 = codingLine[0] <= codingLine[1] < codingLine[2] < ...
+      //     < codingLine[n-1] < codingLine[n] = columns
+      //     -- where 1 <= n <= columns+1
+      for (i = 0; codingLine[i] < columns; ++i) {
 	refLine[i] = codingLine[i];
+      }
       refLine[i] = refLine[i + 1] = columns;
       b1 = 1;
       a0New = codingLine[a0 = 0] = 0;
@@ -1384,68 +1369,93 @@ int CCITTFaxStream::lookChar() {
 	    } while (code3 >= 64);
 	  }
 	  if (code1 > 0 || code2 > 0) {
-	    codingLine[a0 + 1] = a0New + code1;
+	    if (a0New + code1 <= columns) {
+	      codingLine[a0 + 1] = a0New + code1;
+	    } else {
+	      codingLine[a0 + 1] = columns;
+	    }
 	    ++a0;
-	    a0New = codingLine[a0 + 1] = codingLine[a0] + code2;
+	    if (codingLine[a0] + code2 <= columns) {
+	      codingLine[a0 + 1] = codingLine[a0] + code2;
+	    } else {
+	      codingLine[a0 + 1] = columns;
+	    }
 	    ++a0;
-	    while (refLine[b1] <= codingLine[a0] && refLine[b1] < columns)
+	    a0New = codingLine[a0];
+	    while (refLine[b1] <= a0New && refLine[b1] < columns) {
 	      b1 += 2;
+	    }
 	  }
 	  break;
 	case twoDimVert0:
-	  a0New = codingLine[++a0] = refLine[b1];
 	  if (refLine[b1] < columns) {
+	    a0New = codingLine[++a0] = refLine[b1];
 	    ++b1;
-	    while (refLine[b1] <= codingLine[a0] && refLine[b1] < columns)
+	    while (refLine[b1] <= a0New && refLine[b1] < columns) {
 	      b1 += 2;
+	    }
+	  } else {
+	    a0New = codingLine[++a0] = columns;
 	  }
 	  break;
 	case twoDimVertR1:
-	  a0New = codingLine[++a0] = refLine[b1] + 1;
-	  if (refLine[b1] < columns) {
+	  if (refLine[b1] + 1 < columns) {
+	    a0New = codingLine[++a0] = refLine[b1] + 1;
 	    ++b1;
-	    while (refLine[b1] <= codingLine[a0] && refLine[b1] < columns)
+	    while (refLine[b1] <= a0New && refLine[b1] < columns) {
 	      b1 += 2;
+	    }
+	  } else {
+	    a0New = codingLine[++a0] = columns;
 	  }
 	  break;
 	case twoDimVertL1:
-	  if (a0 == 0 || refLine[b1] - 1 > a0New) {
-	  a0New = codingLine[++a0] = refLine[b1] - 1;
-	  --b1;
-	  while (refLine[b1] <= codingLine[a0] && refLine[b1] < columns)
-	    b1 += 2;
+	  if (refLine[b1] - 1 > a0New || (a0 == 0 && refLine[b1] == 1)) {
+	    a0New = codingLine[++a0] = refLine[b1] - 1;
+	    --b1;
+	    while (refLine[b1] <= a0New && refLine[b1] < columns) {
+	      b1 += 2;
+	    }
 	  }
 	  break;
 	case twoDimVertR2:
-	  a0New = codingLine[++a0] = refLine[b1] + 2;
-	  if (refLine[b1] < columns) {
+	  if (refLine[b1] + 2 < columns) {
+	    a0New = codingLine[++a0] = refLine[b1] + 2;
 	    ++b1;
-	    while (refLine[b1] <= codingLine[a0] && refLine[b1] < columns)
+	    while (refLine[b1] <= a0New && refLine[b1] < columns) {
 	      b1 += 2;
+	    }
+	  } else {
+	    a0New = codingLine[++a0] = columns;
 	  }
 	  break;
 	case twoDimVertL2:
-	  if (a0 == 0 || refLine[b1] - 2 > a0New) {
-	  a0New = codingLine[++a0] = refLine[b1] - 2;
-	  --b1;
-	  while (refLine[b1] <= codingLine[a0] && refLine[b1] < columns)
-	    b1 += 2;
+	  if (refLine[b1] - 2 > a0New || (a0 == 0 && refLine[b1] == 2)) {
+	    a0New = codingLine[++a0] = refLine[b1] - 2;
+	    --b1;
+	    while (refLine[b1] <= a0New && refLine[b1] < columns) {
+	      b1 += 2;
+	    }
 	  }
 	  break;
 	case twoDimVertR3:
-	  a0New = codingLine[++a0] = refLine[b1] + 3;
-	  if (refLine[b1] < columns) {
+	  if (refLine[b1] + 3 < columns) {
+	    a0New = codingLine[++a0] = refLine[b1] + 3;
 	    ++b1;
-	    while (refLine[b1] <= codingLine[a0] && refLine[b1] < columns)
+	    while (refLine[b1] <= a0New && refLine[b1] < columns) {
 	      b1 += 2;
+	    }
+	  } else {
+	    a0New = codingLine[++a0] = columns;
 	  }
 	  break;
 	case twoDimVertL3:
-	  if (a0 == 0 || refLine[b1] - 3 > a0New) {
-	  a0New = codingLine[++a0] = refLine[b1] - 3;
-	  --b1;
-	  while (refLine[b1] <= codingLine[a0] && refLine[b1] < columns)
-	    b1 += 2;
+	  if (refLine[b1] - 3 > a0New || (a0 == 0 && refLine[b1] == 3)) {
+	    a0New = codingLine[++a0] = refLine[b1] - 3;
+	    --b1;
+	    while (refLine[b1] <= a0New && refLine[b1] < columns) {
+	      b1 += 2;
+	    }
 	  }
 	  break;
 	case EOF:
@@ -1469,16 +1479,18 @@ int CCITTFaxStream::lookChar() {
 	} while (code3 >= 64);
 	codingLine[a0+1] = codingLine[a0] + code1;
 	++a0;
-	if (codingLine[a0] >= columns)
+	if (codingLine[a0] >= columns) {
 	  break;
+	}
 	code2 = 0;
 	do {
 	  code2 += code3 = getBlackCode();
 	} while (code3 >= 64);
 	codingLine[a0+1] = codingLine[a0] + code2;
 	++a0;
-	if (codingLine[a0] >= columns)
+	if (codingLine[a0] >= columns) {
 	  break;
+	}
       }
     }
 
@@ -1774,7 +1786,7 @@ short CCITTFaxStream::lookBits(int n) {
   return (inputBuf >> (inputBits - n)) & (0xffff >> (16 - n));
 }
 
-GString *CCITTFaxStream::getPSFilter(int psLevel, const char *indent) {
+GString *CCITTFaxStream::getPSFilter(int psLevel, char *indent) {
   GString *s;
   char s1[50];
 
@@ -1815,8 +1827,6 @@ GBool CCITTFaxStream::isBinary(GBool /*last*/) {
   return str->isBinary(gTrue);
 }
 
-#if 0
-
 //------------------------------------------------------------------------
 // DCTStream
 //------------------------------------------------------------------------
@@ -1861,10 +1871,11 @@ static int dctZigZag[64] = {
   63
 };
 
-DCTStream::DCTStream(Stream *strA):
+DCTStream::DCTStream(Stream *strA, GBool colorXformA):
     FilterStream(strA) {
   int i, j;
 
+  colorXform = colorXformA;
   progressive = interleaved = gFalse;
   width = height = 0;
   mcuWidth = mcuHeight = 0;
@@ -1890,20 +1901,8 @@ DCTStream::DCTStream(Stream *strA):
 }
 
 DCTStream::~DCTStream() {
-  int i, j;
-
+  close();
   delete str;
-  if (progressive || !interleaved) {
-    for (i = 0; i < numComps; ++i) {
-      gfree(frameBuf[i]);
-    }
-  } else {
-    for (i = 0; i < numComps; ++i) {
-      for (j = 0; j < mcuHeight; ++j) {
-	gfree(rowBuf[i][j]);
-      }
-    }
-  }
 }
 
 void DCTStream::reset() {
@@ -1917,7 +1916,6 @@ void DCTStream::reset() {
   numQuantTables = 0;
   numDCHuffTables = 0;
   numACHuffTables = 0;
-  colorXform = 0;
   gotJFIFMarker = gFalse;
   gotAdobeMarker = gFalse;
   restartInterval = 0;
@@ -1939,20 +1937,24 @@ void DCTStream::reset() {
     }
     if (compInfo[i].vSample > mcuHeight) {
       mcuHeight = compInfo[i].vSample;
-  }
+    }
   }
   mcuWidth *= 8;
   mcuHeight *= 8;
 
   // figure out color transform
-  if (!gotAdobeMarker && numComps == 3) {
-    if (gotJFIFMarker) {
-      colorXform = 1;
-    } else if (compInfo[0].id == 82 && compInfo[1].id == 71 &&
-	       compInfo[2].id == 66) { // ASCII "RGB"
-      colorXform = 0;
+  if (colorXform == -1) {
+    if (numComps == 3) {
+      if (gotJFIFMarker) {
+	colorXform = 1;
+      } else if (compInfo[0].id == 82 && compInfo[1].id == 71 &&
+		 compInfo[2].id == 66) { // ASCII "RGB"
+	colorXform = 0;
+      } else {
+	colorXform = 1;
+      }
     } else {
-      colorXform = 1;
+      colorXform = 0;
     }
   }
 
@@ -2002,6 +2004,20 @@ void DCTStream::reset() {
   }
 }
 
+void DCTStream::close() {
+  int i, j;
+
+  for (i = 0; i < 4; ++i) {
+    for (j = 0; j < 32; ++j) {
+      gfree(rowBuf[i][j]);
+      rowBuf[i][j] = NULL;
+    }
+    gfree(frameBuf[i]);
+    frameBuf[i] = NULL;
+  }
+  FilterStream::close();
+}
+
 int DCTStream::getChar() {
   int c;
 
@@ -2926,11 +2942,11 @@ GBool DCTStream::readBaselineSOF() {
   width = read16();
   numComps = str->getChar();
   if (numComps <= 0 || numComps > 4) {
+    error(getPos(), "Bad number of components in DCT stream");
     numComps = 0;
-    error(getPos(), "Bad number of components in DCT stream", prec);
     return gFalse;
   }
- if (prec != 8) {
+  if (prec != 8) {
     error(getPos(), "Bad DCT precision %d", prec);
     return gFalse;
   }
@@ -2957,8 +2973,8 @@ GBool DCTStream::readProgressiveSOF() {
   width = read16();
   numComps = str->getChar();
   if (numComps <= 0 || numComps > 4) {
-    numComps = 0;
     error(getPos(), "Bad number of components in DCT stream");
+    numComps = 0;
     return gFalse;
   }
   if (prec != 8) {
@@ -2984,8 +3000,8 @@ GBool DCTStream::readScanInfo() {
   length = read16() - 2;
   scanInfo.numComps = str->getChar();
   if (scanInfo.numComps <= 0 || scanInfo.numComps > 4) {
-    scanInfo.numComps = 0;
     error(getPos(), "Bad number of components in DCT stream");
+    scanInfo.numComps = 0;
     return gFalse;
   }
   --length;
@@ -3047,14 +3063,14 @@ GBool DCTStream::readQuantTables() {
       if (prec) {
 	quantTables[index][dctZigZag[i]] = read16();
       } else {
-      quantTables[index][dctZigZag[i]] = str->getChar();
+	quantTables[index][dctZigZag[i]] = str->getChar();
       }
     }
     if (prec) {
       length -= 129;
     } else {
-    length -= 65;
-  }
+      length -= 65;
+    }
   }
   return gTrue;
 }
@@ -3072,12 +3088,12 @@ GBool DCTStream::readHuffmanTables() {
   while (length > 0) {
     index = str->getChar();
     --length;
-    if ((index & ~0x10) >= 4 || (index & ~0x10) < 0) {
+    if ((index & 0x0f) >= 4) {
       error(getPos(), "Bad DCT Huffman table");
       return gFalse;
     }
     if (index & 0x10) {
-      index &= 0x03;
+      index &= 0x0f;
       if (index >= numACHuffTables)
 	numACHuffTables = index+1;
       tbl = &acHuffTables[index];
@@ -3214,7 +3230,7 @@ int DCTStream::read16() {
   return (c1 << 8) + c2;
 }
 
-GString *DCTStream::getPSFilter(int psLevel, const char *indent) {
+GString *DCTStream::getPSFilter(int psLevel, char *indent) {
   GString *s;
 
   if (psLevel < 2) {
@@ -3227,12 +3243,10 @@ GString *DCTStream::getPSFilter(int psLevel, const char *indent) {
   return s;
 }
 
-GBool DCTStream::isBinary(GBool last) {
+GBool DCTStream::isBinary(GBool /*last*/) {
   return str->isBinary(gTrue);
 }
 
-#endif
-
 //------------------------------------------------------------------------
 // FlateStream
 //------------------------------------------------------------------------
@@ -3880,14 +3894,15 @@ FlateStream::FlateStream(Stream *strA, int predictor, int columns,
   }
   litCodeTab.codes = NULL;
   distCodeTab.codes = NULL;
+  memset(buf, 0, flateWindow);
 }
 
 FlateStream::~FlateStream() {
   if (litCodeTab.codes != fixedLitCodeTab.codes) {
-  gfree(litCodeTab.codes);
+    gfree(litCodeTab.codes);
   }
   if (distCodeTab.codes != fixedDistCodeTab.codes) {
-  gfree(distCodeTab.codes);
+    gfree(distCodeTab.codes);
   }
   if (pred) {
     delete pred;
@@ -3977,7 +3992,7 @@ int FlateStream::getRawChar() {
   return c;
 }
 
-GString *FlateStream::getPSFilter(int psLevel, const char *indent) {
+GString *FlateStream::getPSFilter(int psLevel, char *indent) {
   GString *s;
 
   if (psLevel < 3 || pred) {
@@ -4066,11 +4081,11 @@ GBool FlateStream::startBlock() {
 
   // free the code tables from the previous block
   if (litCodeTab.codes != fixedLitCodeTab.codes) {
-  gfree(litCodeTab.codes);
+    gfree(litCodeTab.codes);
   }
   litCodeTab.codes = NULL;
   if (distCodeTab.codes != fixedDistCodeTab.codes) {
-  gfree(distCodeTab.codes);
+    gfree(distCodeTab.codes);
   }
   distCodeTab.codes = NULL;
 
@@ -4392,7 +4407,7 @@ void ASCIIHexEncoder::reset() {
 }
 
 GBool ASCIIHexEncoder::fillBuf() {
-  static const char *hex = "0123456789abcdef";
+  static char *hex = "0123456789abcdef";
   int c;
 
   if (eof) {
@@ -4438,30 +4453,34 @@ void ASCII85Encoder::reset() {
 }
 
 GBool ASCII85Encoder::fillBuf() {
-  Gulong t;
+  Guint t;
   char buf1[5];
-  int c;
+  int c0, c1, c2, c3;
   int n, i;
 
-  if (eof)
+  if (eof) {
     return gFalse;
-  t = 0;
-  for (n = 0; n < 4; ++n) {
-    if ((c = str->getChar()) == EOF)
-      break;
-    t = (t << 8) + c;
   }
+  c0 = str->getChar();
+  c1 = str->getChar();
+  c2 = str->getChar();
+  c3 = str->getChar();
   bufPtr = bufEnd = buf;
-  if (n > 0) {
-    if (n == 4 && t == 0) {
-      *bufEnd++ = 'z';
-      if (++lineLen == 65) {
-	*bufEnd++ = '\n';
-	lineLen = 0;
-      }
+  if (c3 == EOF) {
+    if (c0 == EOF) {
+      n = 0;
+      t = 0;
     } else {
-      if (n < 4)
-	t <<= 8 * (4 - n);
+      if (c1 == EOF) {
+	n = 1;
+	t = c0 << 24;
+      } else if (c2 == EOF) {
+	n = 2;
+	t = (c0 << 24) | (c1 << 16);
+      } else {
+	n = 3;
+	t = (c0 << 24) | (c1 << 16) | (c2 << 8);
+      }
       for (i = 4; i >= 0; --i) {
 	buf1[i] = (char)(t % 85 + 0x21);
 	t /= 85;
@@ -4474,13 +4493,32 @@ GBool ASCII85Encoder::fillBuf() {
 	}
       }
     }
-  }
-  if (n < 4) {
     *bufEnd++ = '~';
     *bufEnd++ = '>';
     eof = gTrue;
+  } else {
+    t = (c0 << 24) | (c1 << 16) | (c2 << 8) | c3;
+    if (t == 0) {
+      *bufEnd++ = 'z';
+      if (++lineLen == 65) {
+	*bufEnd++ = '\n';
+	lineLen = 0;
+      }
+    } else {
+      for (i = 4; i >= 0; --i) {
+	buf1[i] = (char)(t % 85 + 0x21);
+	t /= 85;
+      }
+      for (i = 0; i <= 4; ++i) {
+	*bufEnd++ = buf1[i];
+	if (++lineLen == 65) {
+	  *bufEnd++ = '\n';
+	  lineLen = 0;
+	}
+      }
+    }
   }
-  return bufPtr < bufEnd;
+  return gTrue;
 }
 
 //------------------------------------------------------------------------
