@@ -13,10 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
-
-   $Id: kscandevice.cpp,v 1.43 2003/08/25 14:05:01 freitag Exp $
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <stdlib.h>
@@ -37,6 +35,7 @@
 #include <klocale.h>
 #include <kglobal.h>
 #include <kconfig.h>
+#include <kstandarddirs.h>
 
 #include <unistd.h>
 #include "kgammatable.h"
@@ -45,12 +44,33 @@
 #include "kscanoption.h"
 #include "kscanoptset.h"
 #include "devselector.h"
+#include "imgscaninfo.h"
+
 #include <ksimpleconfig.h>
 
 #define MIN_PREVIEW_DPI 75
 #define UNDEF_SCANNERNAME I18N_NOOP( "undefined" )
 #define MAX_PROGRESS 100
 
+/* ---------------------------------------------------------------------------
+   Private class for KScanDevice
+   ------------------------------------------------------------------------- */
+class KScanDevice::KScanDevicePrivate
+
+{
+public:
+    KScanDevicePrivate()
+	: currScanResolutionX(0),
+	  currScanResolutionY(0)
+	{
+	    
+	}
+
+    int currScanResolutionX, currScanResolutionY;
+    
+};
+
+
 /* ---------------------------------------------------------------------------
 
    ------------------------------------------------------------------------- */
@@ -148,6 +168,8 @@ KScanDevice::KScanDevice( QObject *parent )
 {
     SANE_Status sane_stat = sane_init(NULL, NULL );
 
+    d = new KScanDevicePrivate();
+    
     option_dic = new QAsciiDict<int>;
     option_dic->setAutoDelete( true );
     gui_elements.setAutoDelete( true );
@@ -155,10 +177,10 @@ KScanDevice::KScanDevice( QObject *parent )
     scanner_initialised = false;  /* stays false until openDevice. */
     scanStatus = SSTAT_SILENT;
 
-    data         = 0; /* temporar image data buffer while scanning */
-    sn           = 0; /* socket notifier for async scanning        */
-    img          = 0; /* temporar image to scan in                 */
-    storeOptions = 0; /* list of options to store during preview   */
+    data         = 0; /* temporary image data buffer while scanning */
+    sn           = 0; /* socket notifier for async scanning         */
+    img          = 0; /* temporary image to scan in                 */
+    storeOptions = 0; /* list of options to store during preview    */
     overall_bytes = 0;
     rest_bytes = 0;
     pixel_x = 0;
@@ -288,7 +310,7 @@ QString KScanDevice::getScannerName(const QCString& name) const
   else
   {
      scanner = scannerDevices[ name ];
-     ret = "";
+     ret = QString::null;
   }
 
   if( scanner ) {
@@ -643,6 +665,43 @@ void KScanDevice::slStopScanning( void )
     scanStatus = SSTAT_STOP_NOW;
 }
 
+
+const QString KScanDevice::previewFile()
+{
+   QString dir = (KGlobal::dirs())->saveLocation( "data", "ScanImages", true );
+   if( !dir.endsWith("/") )
+      dir += "/";
+
+   QString fname = dir + QString::fromLatin1(".previews/");
+   QString sname( getScannerName(shortScannerName()) );
+   sname.replace( '/', "_");
+
+   return fname+sname;
+}
+
+QImage KScanDevice::loadPreviewImage()
+{
+   const QString prevFile = previewFile();
+   kdDebug(29000) << "Loading preview from file " << prevFile << endl;
+
+   QImage image;
+   image.load( prevFile );
+
+   return image;
+}
+
+bool KScanDevice::savePreviewImage( const QImage &image )
+{
+   if( image.isNull() )
+	return false;
+
+   const QString prevFile = previewFile();
+   kdDebug(29000) << "Saving preview to file " << prevFile << endl;
+
+   return image.save( prevFile, "BMP" );
+}
+
+
 KScanStat KScanDevice::acquirePreview( bool forceGray, int dpi )
 {
    double min, max, q;
@@ -712,12 +771,17 @@ KScanStat KScanDevice::acquirePreview( bool forceGray, int dpi )
 
    if( dpi == 0 )
    {
-      /* No resolution argument */
-      if( ! res.getRange( &min, &max, &q ) )
-      {
-	 kdDebug(29000) << "Could not retrieve resolution range!" << endl;
-	 min = 75.0; // Hope that every scanner can 75
-      }
+       /* No resolution argument */
+       if( ! res.getRange( &min, &max, &q ) )
+       {
+	   if( ! res.getRangeFromList ( &min, &max, &q ) )
+	   {
+	       kdDebug(29000) << "Could not retrieve resolution range!" << endl;
+	       min = 75.0; // Hope that every scanner can 75
+	   }
+       }
+       kdDebug(29000) << "Minimum Range: " << min << ", Maximum Range: " << max << endl;
+
       if( min > MIN_PREVIEW_DPI )
 	 set_dpi = (int) min;
       else
@@ -725,13 +789,16 @@ KScanStat KScanDevice::acquirePreview( bool forceGray, int dpi )
    }
 
    /* Set scan resolution for preview. */
-   if( optionExists( SANE_NAME_SCAN_Y_RESOLUTION ) )
+   if( !optionExists( SANE_NAME_SCAN_Y_RESOLUTION ) )
+      d->currScanResolutionY = 0;
+   else
    {
       KScanOption yres ( SANE_NAME_SCAN_Y_RESOLUTION );
       /* if active ? */
       storeOptions->backupOption( yres );
       yres.set( set_dpi );
       apply( &yres );
+      yres.get( &d->currScanResolutionY );
 
       /* Resolution bind switch ? */
       if( optionExists( SANE_NAME_RESOLUTION_BIND ) )
@@ -747,6 +814,11 @@ KScanStat KScanDevice::acquirePreview( bool forceGray, int dpi )
    res.set( set_dpi );
    apply( &res );
 
+   /* Store the resulting preview for additional image information */
+   res.get( &d->currScanResolutionX );
+
+   if ( d->currScanResolutionY == 0 )
+      d->currScanResolutionY = d->currScanResolutionX;
 
    /* Start scanning */
    KScanStat stat = acquire_data( true );
@@ -848,6 +920,18 @@ KScanStat KScanDevice::acquire( const QString& filename )
  	        kdDebug(29000) << "Option <" << so->getName() << "> is not active !" << endl;
  	    }
  	}
+
+	/** Scan Resolution should always exist. **/
+	KScanOption res( SANE_NAME_SCAN_RESOLUTION );
+	res.get( &d->currScanResolutionX );
+        if ( !optionExists( SANE_NAME_SCAN_Y_RESOLUTION ) )
+           d->currScanResolutionY = d->currScanResolutionX;
+        else
+        {
+           KScanOption yres( SANE_NAME_SCAN_Y_RESOLUTION );
+           yres.get( &d->currScanResolutionY );
+        }
+
 	return( acquire_data( false ));
     }
     else
@@ -857,9 +941,13 @@ KScanStat KScanDevice::acquire( const QString& filename )
 	if( file.exists() )
 	{
 	     QImage i;
+	     ImgScanInfo info;
 	     if( i.load( filename ))
 	     {
-                  emit( sigNewImage( &i ));
+		info.setXResolution(i.dotsPerMeterX()); // TODO: *2.54/100
+		info.setYResolution(i.dotsPerMeterY()); // TODO: *2.54/100
+		info.setScannerName(filename);
+		emit( sigNewImage( &i, &info ));
 	     }
 	}
     }
@@ -977,7 +1065,7 @@ KScanStat KScanDevice::acquire_data( bool isPreview )
    if( stat == KSCAN_OK )
    {
       /* new buffer for scanning one line */
-      if(data) delete data;
+      if(data) delete [] data;
       data = new SANE_Byte[ sane_scan_param.bytes_per_line +4 ];
       if( ! data ) stat = KSCAN_ERR_MEMORY;
    }
@@ -1079,58 +1167,64 @@ void KScanDevice::loadOptionSet( KScanOptSet *optSet )
 
 void KScanDevice::slScanFinished( KScanStat status )
 {
-   // clean up
-   if( sn ) {
-      sn->setEnabled( false );
-      delete sn;
-      sn = 0;
-   }
+    // clean up
+    if( sn ) {
+	sn->setEnabled( false );
+	delete sn;
+	sn = 0;
+    }
 
-   emit( sigScanProgress( MAX_PROGRESS ));
+    emit( sigScanProgress( MAX_PROGRESS ));
 
-   kdDebug(29000) << "Slot ScanFinished hit with status " <<  status << endl;
+    kdDebug(29000) << "Slot ScanFinished hit with status " <<  status << endl;
 
-   if( data )
-   {
-      delete data;
-      data = 0;
-   }
+    if( data )
+    {
+	delete[] data;
+	data = 0;
+    }
 
-   const QString qq;
-   KScanOption *so;
-   (void) so;
+    if( status == KSCAN_OK && img )
+    {
+	ImgScanInfo info;
+	info.setXResolution(d->currScanResolutionX);
+	info.setYResolution(d->currScanResolutionY);
+	info.setScannerName(shortScannerName());
 
-   if( status == KSCAN_OK )
-   {
-      if( scanningPreview )
-      {
-	 kdDebug(29000) << "Scanning a preview !" << endl;
-	 emit( sigNewPreview( img ));
+	// put the resolution also into the image itself
+	img->setDotsPerMeterX(static_cast<int>(d->currScanResolutionX / 0.0254 + 0.5));
+	img->setDotsPerMeterY(static_cast<int>(d->currScanResolutionY / 0.0254 + 0.5));
 
-	 /* The old settings need to be redefined */
-	 loadOptionSet( storeOptions );
-      }
-      else
-      {
-	 emit( sigNewImage( img ));
-      }
-   }
+	if( scanningPreview )
+	{
+	    kdDebug(29000) << "Scanning a preview !" << endl;
+	    savePreviewImage(*img);
+	    emit( sigNewPreview( img, &info ));
+
+	    /* The old settings need to be redefined */
+	    loadOptionSet( storeOptions );
+	}
+	else
+	{
+	    emit( sigNewImage( img, &info ));
+	}
+    }
 
 
-   sane_cancel(scanner_handle);
+    sane_cancel(scanner_handle);
 
-   /* This follows after sending the signal */
-   if( img )
-   {
-      delete img;
-      img = 0;
-   }
+    /* This follows after sending the signal */
+    if( img )
+    {
+	delete img;
+	img = 0;
+    }
 
-   /* delete the socket notifier */
-   if( sn ) {
-      delete( sn );
-      sn = 0;
-   }
+    /* delete the socket notifier */
+    if( sn ) {
+	delete( sn );
+	sn = 0;
+    }
 
 }
 
@@ -1139,7 +1233,7 @@ void KScanDevice::slScanFinished( KScanStat status )
  * to the qimage.
  * The function needs:
  * QImage img valid
- * the data-buffer  set to a approbiate size
+ * the data-buffer  set to a appropriate size
  **/
 
 void KScanDevice::doProcessABlock( void )
@@ -1163,7 +1257,6 @@ void KScanDevice::doProcessABlock( void )
      int       green = 0;
      int       blue = 0;
 
-
      if( sane_stat != SANE_STATUS_GOOD )
      {
 	/** any other error **/
@@ -1296,8 +1389,8 @@ void KScanDevice::doProcessABlock( void )
 
 	if( (sane_scan_param.lines > 0) && (sane_scan_param.lines * pixel_y> 0) )
 	{
-	   int progress =  (int)((double)MAX_PROGRESS / (double) sane_scan_param.lines *
-				 (double)pixel_y);
+	   int progress =  (int)(((double)MAX_PROGRESS) / sane_scan_param.lines *
+				  pixel_y);
 	   if( progress < MAX_PROGRESS)
 	      emit( sigScanProgress( progress));
 	}
